@@ -16,13 +16,11 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtegl.c,v 1.15 2004/04/20 05:16:56 nalin Exp $"
 
 #include "../config.h"
 
 #include <gtk/gtk.h>
 
-#if GTK_CHECK_VERSION(2,2,0)
 #ifndef X_DISPLAY_MISSING
 #ifdef HAVE_GL
 
@@ -32,7 +30,6 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <GL/gl.h>
-#include <GL/glu.h>
 #include <GL/glx.h>
 #include <X11/Xutil.h>
 #include "buffer.h"
@@ -45,6 +42,7 @@
 struct _vte_gl_data
 {
 	XVisualInfo *visual_info;
+	Display *display;
 	GLXContext context;
 	GdkColor color;
 	GdkPixbuf *bgpixbuf;
@@ -69,10 +67,10 @@ _vte_gl_check(struct _vte_draw *draw, GtkWidget *widget)
 	int error, event;
 	gboolean direct;
 
-	gdisplay = gdk_display_get_default();
-	display = gdk_x11_display_get_xdisplay(gdisplay);
-	gscreen = gdk_screen_get_default();
-	screen = gdk_x11_screen_get_screen_number(gscreen);
+	gdisplay = gtk_widget_get_display (widget);
+	display = gdk_x11_display_get_xdisplay (gdisplay);
+	gscreen = gtk_widget_get_screen (widget);
+	screen = gdk_x11_screen_get_screen_number (gscreen);
 
 	/* Check for GLX. */
 	if (!glXQueryExtension(display, &error, &event)) {
@@ -123,13 +121,15 @@ _vte_gl_create(struct _vte_draw *draw, GtkWidget *widget)
 	int screen;
 	gboolean direct;
 
-	draw->impl_data = g_malloc(sizeof(struct _vte_gl_data));
+	draw->impl_data = g_slice_new(struct _vte_gl_data);
 	data = (struct _vte_gl_data*) draw->impl_data;
 
-	gdisplay = gdk_display_get_default();
-	display = gdk_x11_display_get_xdisplay(gdisplay);
-	gscreen = gdk_screen_get_default();
-	screen = gdk_x11_screen_get_screen_number(gscreen);
+	gdisplay = gtk_widget_get_display (widget);
+	display = gdk_x11_display_get_xdisplay (gdisplay);
+	gscreen = gtk_widget_get_screen (widget);
+	screen = gdk_x11_screen_get_screen_number (gscreen);
+
+	data->display = display;
 
 	data->visual_info = glXChooseVisual(display, screen, attributes);
 	if (data->visual_info == NULL) {
@@ -142,11 +142,6 @@ _vte_gl_create(struct _vte_draw *draw, GtkWidget *widget)
 		g_error("Unable to create a GLX context.\n");
 	}
 
-	direct = glXIsDirect(display, data->context);
-	if (!direct) {
-		g_error("Unable to create a direct GLX context.\n");
-	}
-
 	data->color.red = 0;
 	data->color.green = 0;
 	data->color.blue = 0;
@@ -163,38 +158,22 @@ static void
 _vte_gl_destroy(struct _vte_draw *draw)
 {
 	struct _vte_gl_data *data;
-	GdkDisplay *gdisplay;
-	Display *display;
-	GdkScreen *gscreen;
-	int screen;
 
 	data = (struct _vte_gl_data*) draw->impl_data;
-	gdisplay = gdk_display_get_default();
-	display = gdk_x11_display_get_xdisplay(gdisplay);
-	gscreen = gdk_screen_get_default();
-	screen = gdk_x11_screen_get_screen_number(gscreen);
 
 	_vte_buffer_free(data->buffer);
-	data->buffer = NULL;
 
 	_vte_glyph_cache_free(data->cache);
-	data->cache = NULL;
 
-	if (GDK_IS_PIXBUF(data->bgpixbuf)) {
-		g_object_unref(G_OBJECT(data->bgpixbuf));
+	if (data->bgpixbuf != NULL) {
+		g_object_unref(data->bgpixbuf);
 	}
-	data->bgpixbuf = NULL;
 
-	glXMakeCurrent(display, None, data->context);
+	glXMakeCurrent(data->display, None, data->context);
 
-	glXDestroyContext(display, data->context);
-	data->context = NULL;
+	glXDestroyContext(data->display, data->context);
 
-	data->scrollx = data->scrolly = 0;
-
-	memset(&data->color, 0, sizeof(data->color));
-
-	g_free(draw->impl_data);
+	g_slice_free(struct _vte_gl_data, draw->impl_data);
 }
 
 static GdkVisual *
@@ -223,22 +202,19 @@ static void
 _vte_gl_start(struct _vte_draw *draw)
 {
 	struct _vte_gl_data *data;
-	GdkDisplay *gdisplay;
-	Display *display;
 	gint width, height;
 
 	data = (struct _vte_gl_data*) draw->impl_data;
-	gdisplay = gdk_display_get_default();
-	display = gdk_x11_display_get_xdisplay(gdisplay);
 
 	data->glwindow = gdk_x11_drawable_get_xid(draw->widget->window);
-	gdk_drawable_get_size(draw->widget->window, &width, &height);
+	width = draw->widget->allocation.width;
+	height = draw->widget->allocation.height;
 
-	glXMakeCurrent(display, data->glwindow, data->context);
+	glXMakeCurrent(data->display, data->glwindow, data->context);
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
-	gluOrtho2D(0, width - 1, height - 1, 0);
-	glViewport(0, height - 1, width, -height);
+	glOrtho(0, width, height, 0, -1, 1);
+	glViewport(0, height, width, -height);
 	glEnable(GL_BLEND);
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 }
@@ -247,21 +223,17 @@ static void
 _vte_gl_end(struct _vte_draw *draw)
 {
 	struct _vte_gl_data *data;
-	GdkDisplay *gdisplay;
-	Display *display;
 
 	data = (struct _vte_gl_data*) draw->impl_data;
-	gdisplay = gdk_display_get_default();
-	display = gdk_x11_display_get_xdisplay(gdisplay);
 
-	glXMakeCurrent(display, data->glwindow, data->context);
-	glXSwapBuffers(display, data->glwindow);
+	glXMakeCurrent(data->display, data->glwindow, data->context);
+	glXSwapBuffers(data->display, data->glwindow);
 
 	data->glwindow = -1;
 }
 
 static void
-_vte_gl_set_background_color(struct _vte_draw *draw, GdkColor *color)
+_vte_gl_set_background_color(struct _vte_draw *draw, GdkColor *color, guint16 opacity)
 {
 	struct _vte_gl_data *data;
 
@@ -279,12 +251,16 @@ _vte_gl_set_background_image(struct _vte_draw *draw,
 {
 	struct _vte_gl_data *data;
 	GdkPixbuf *bgpixbuf;
+	GdkScreen *screen;
+
+	screen = gtk_widget_get_screen(draw->widget);
 
 	data = (struct _vte_gl_data*) draw->impl_data;
-	bgpixbuf = vte_bg_get_pixbuf(vte_bg_get(), type, pixbuf, file,
+	bgpixbuf = vte_bg_get_pixbuf(vte_bg_get_for_screen(screen),
+				     type, pixbuf, file,
 				     tint, saturation);
-	if (GDK_IS_PIXBUF(data->bgpixbuf)) {
-		g_object_unref(G_OBJECT(data->bgpixbuf));
+	if (data->bgpixbuf != NULL) {
+		g_object_unref(data->bgpixbuf);
 	}
 	data->bgpixbuf = bgpixbuf;
 }
@@ -292,8 +268,6 @@ _vte_gl_set_background_image(struct _vte_draw *draw,
 static void
 _vte_gl_clear(struct _vte_draw *draw, gint x, gint y, gint width, gint height)
 {
-	GdkDisplay *gdisplay;
-	Display *display;
 	struct _vte_gl_data *data;
 	long xstop, ystop, i, j;
 	int pixbufw, pixbufh, w, h, channels, stride;
@@ -301,12 +275,10 @@ _vte_gl_clear(struct _vte_draw *draw, gint x, gint y, gint width, gint height)
 	guchar *pixels;
 
 	data = (struct _vte_gl_data*) draw->impl_data;
-	gdisplay = gdk_display_get_default();
-	display = gdk_x11_display_get_xdisplay(gdisplay);
 
-	glXMakeCurrent(display, data->glwindow, data->context);
+	glXMakeCurrent(data->display, data->glwindow, data->context);
 
-	if (GDK_IS_PIXBUF(data->bgpixbuf)) {
+	if (data->bgpixbuf != NULL) {
 		pixbufw = gdk_pixbuf_get_width(data->bgpixbuf);
 		pixbufh = gdk_pixbuf_get_height(data->bgpixbuf);
 	} else {
@@ -457,8 +429,6 @@ _vte_gl_draw_text(struct _vte_draw *draw,
 		  struct _vte_draw_text_request *requests, gsize n_requests,
 		  GdkColor *color, guchar alpha)
 {
-	GdkDisplay *gdisplay;
-	Display *display;
 	struct _vte_gl_data *data;
 	const struct _vte_glyph *glyph;
 	guint16 a, r, g, b;
@@ -466,10 +436,8 @@ _vte_gl_draw_text(struct _vte_draw *draw,
 	guchar *pixels;
 
 	data = (struct _vte_gl_data*) draw->impl_data;
-	gdisplay = gdk_display_get_default();
-	display = gdk_x11_display_get_xdisplay(gdisplay);
 
-	glXMakeCurrent(display, data->glwindow, data->context);
+	glXMakeCurrent(data->display, data->glwindow, data->context);
 
 	r = color->red >> 8;
 	g = color->green >> 8;
@@ -556,15 +524,11 @@ _vte_gl_rectangle(struct _vte_draw *draw,
 		  gint x, gint y, gint width, gint height,
 		  GdkColor *color, guchar alpha)
 {
-	GdkDisplay *gdisplay;
-	Display *display;
 	struct _vte_gl_data *data;
 
 	data = (struct _vte_gl_data*) draw->impl_data;
-	gdisplay = gdk_display_get_default();
-	display = gdk_x11_display_get_xdisplay(gdisplay);
 
-	glXMakeCurrent(display, data->glwindow, data->context);
+	glXMakeCurrent(data->display, data->glwindow, data->context);
 
 	glColor4us(color->red, color->green, color->blue,
 		   (alpha == VTE_DRAW_OPAQUE) ? 0xffff : (alpha << 8));
@@ -604,8 +568,8 @@ _vte_gl_set_scroll(struct _vte_draw *draw, gint x, gint y)
 	data->scrolly = y;
 }
 
-struct _vte_draw_impl _vte_draw_gl = {
-	"VteGL", "VTE_USE_GL",
+const struct _vte_draw_impl _vte_draw_gl = {
+	"GL",
 	_vte_gl_check,
 	_vte_gl_create,
 	_vte_gl_destroy,
@@ -616,6 +580,7 @@ struct _vte_draw_impl _vte_draw_gl = {
 	_vte_gl_set_background_color,
 	_vte_gl_set_background_image,
 	TRUE,
+	NULL,
 	_vte_gl_clear,
 	_vte_gl_set_text_font,
 	_vte_gl_get_text_width,
@@ -632,4 +597,3 @@ struct _vte_draw_impl _vte_draw_gl = {
 
 #endif
 #endif
-#endif
